%value of delta
d = 0.9;

%setting seed and generator for random number generator
rng(2, "twister");

%this is for purposes of defining the distribution of the decay shocks used
%for the simulation
pd = makedist('Beta','a',5,'b',1);
%pd = makedist('Triangular','a',0,'b',0.8,'c',1);
%pd = makedist('Uniform');
m  = 0.99*mean(pd);


%calculates a sequence of policies starting at a = 0.5 with a
%Beta(1,5)-distributed shock distribution and T = 1
%G1 is the array of policies, and F1 the policy increments corresponding to
%the sequence
[G1, H1, J1, s1, C1, B1, K1, M1] = recur(1/(1-d*m), 0, 1/(1-d*m), d);
F1 = abs(diff(G1));
G1(101:end);
F1(101:end);


%above, but for T = 2
[G2, H2, J2, s2, C2, B2, K2, M2] = recur(H1, J1, s1, d);
F2 = abs(diff(G2));
G2(101:end);
F2(101:end);


%above, but for T = 3
[G3, H3, J3, s3, C3, B3, K3, M3] = recur(H2, J2, s2, d);
F3 = abs(diff(G3));
G3(101:end);
F3(101:end);


%above, but for T = 4
[G4, H4, J4, s4, C4, B4, K4, M4] = recur(H3, J3, s3, d);
F4 = abs(diff(G4));
G4(101:end);
F4(101:end);


%plots the distributions of policies and increments simulated above
%also calculates the mean and variance of the increment distribution

Ctrs = [0.005:0.01:0.28];

figure(1)
histfit(G1, 20)

figure(2)
hist(F1, Ctrs, 'Facecolor', 'g')
mean(F1)
var(F1)

figure(3)
histfit(G2, 20)

figure(4)
hist(F2,Ctrs, 'Facecolor', 'g')
mean(F2)
var(F2)

figure(5)
histfit(G3, 20)


figure(6)
hist(F3, Ctrs, 'Facecolor', 'g')
mean(F3)
var(F3)

figure(7)
histfit(G4, 20)

figure(8)
hist(F4, Ctrs, 'Facecolor', 'g')
mean(F4)
var(F4)



function delta1 = d1(e, d, s, H)
    delta1 = ((1 - d)*s - e)/H;
end

%the recursive function with myopic voters
function [G, H1, J1, s1, C, B, K, M] = recur(H, J, s, d)

%setting seed and generator for random number generator
rng(2, "twister");

N = 10000;
max = 30;

%generating random draws from appropriate shock distribution
pd = makedist('Beta','a',5,'b',1);
%pd = makedist('Triangular','a',0,'b',0.8,'c',1);
%pd = makedist('Uniform');


D = 0.99*random(pd, max, N);



%initializing matrix used to calculate partial products
Dp = ones(max, N);


%creating the sample shock partial products
for i = 1:max
    for j = 1:N
        for k = 1:i
            Dp(i,j) = Dp(i,j)*D(k,j);
        end
    end
end

%determines how fine the grid for 'stopping efficiencies' is
precision = 0.001;

e = 0:precision:(1-d)*s;
num = size(e);

a = 0.5;



B = ones(num);
C = ones(num);
K = zeros(num);
M = zeros(num);

num = num(2);

for k = 1:num
    Cm1   = ones(1, N);
    Bm1   = zeros(1, N);

    prod  = ones(max, N);
    prod2 = ones(max, N);
    for i = 2:max
        for j = 1:N
            prod(i,j)  = prod(i-1,j)*(1 - d1(Dp(i-1,j)*e(k),d, s, H));
            prod2(i,j) = prod(i-1,j)*((1 - .5*d1(Dp(i-1,j)*e(k), d, s, H))*d1(Dp(i-1,j)*e(k), d, s, H) - (1-d)*J/(d*H));
        end
    end

    for j = 1:N
        for i = 2:max 
            Cm1(1,j) = Cm1(1,j) + d.^(i-1)*prod(i,j)*Dp(i-1,j);
            Bm1(1,j) = Bm1(1,j) + d.^(i-1)*prod2(i,j);
        end
    end

C(1,k)  = sum(Cm1, 2)/N*e(k)/H;
B(1,k)  = sum(Bm1, 2)/N;

K(1,k)  = C(1,k)*(1 - d1(e(k), d, s, H));
M(1,k)  = B(1,k)*(1 - d1(e(k), d, s, H)) + (1 - .5*d1(e(k), d, s, H))*d1(e(k), d, s, H);

end


sampleH = zeros(1,N);
sampleJ = zeros(1,N);
sampleS = zeros(1,N);
      G = a*ones(1,N);

%setting seed and generator for random number generator
rng(2, "twister");
     r1 = rand(N);
     
     
for j = 1:N
    eta = 1;
    i   = 1;
    while Dp(i,j) > (1-d)*s
        eta = eta + d.^i*Dp(i,j);
        i   = i + 1;
    end
    
    index = floor(Dp(i,j)/precision) + 1;
    
    sampleH(j) = eta + d.^i*H*K(1, index);
    sampleJ(j) = d.^i*(H*M(1, index) + J);
    sampleS(j) = eta + d.^i*s;
    
    inc          = 1;
    winProb      = C(1, index)*G(j) + B(1, index) + d1(e(index), d, s, H)*0.5;
        
    if (r1(j) < winProb)
       inc = 0;
    end
        
    G(j+1) = C(index)*G(j) + B(index) + inc*d1(e(index), d, s, H);
 
   
end

s1 = sum(sampleS)/N;
H1 = sum(sampleH)/N;
J1 = sum(sampleJ)/N;

end











